#!/bin/bash
# IBM WebSphere DataPower Appliances
# Copyright IBM Corporation 2010
# * Unsupported *
# sshd version checker script

# Check the current version of openssh

# if version is ok, then move on.
WD=`dirname $0`
WD=`cd $WD;pwd`

export UNSUPPORTED_VER="OpenSSH_4.2p1"
function check_sshd_permit_tunnel ()
{
	FOUND=`awk 'BEGIN{res="no";} /PermitTunnel/ { if ($1 == "PermitTunnel") { if ($2 == "yes"){ res="yes"; }} }END{print res}' $1`
	if [ "X$FOUND" == "Xno" ];then
		echo "ERROR: PermitTunnel yes  -- is not configured in $1"
		exit 1
	fi
}

function check_config_tun ()
{
	# first do modprobe tun....
	modprobe tun
	TUN=`grep 't tun_net_open' /proc/kallsyms`
	if [ "X$TUN" == "X" ]; then
		echo "ERROR: The running kernel does not support TUN devices"
		echo "ERROR: Use a kernel that has CONFIG_TUN=y"
		exit 1;
	fi

	if [ ! -c "/dev/net/tun" ];then
		echo "ERROR: /dev/net/tun is not present. It is needed by Secure Cloud Connector"
		exit 1;
	fi

	echo "The running kernel supports TUN devices"

}

function check_sshd_version ()
{
	SSHD_BIN=$1
	SSHD_CONFIG=$2

	if [ -f $SSHD_BIN ] ; then
		CURR_VER=`$SSHD_BIN -v  2>&1 | grep OpenSSH | awk -F ' |,' '{print $1}'`
		OLDER_VER=`echo -e "$CURR_VER\n$UNSUPPORTED_VER"  | sort | head -1`
		if [ "X$CURR_VER" != "X$OLDER_VER" ]; then
        		echo "OpenSSH version $CURR_VER  at $SSHD_BIN is supported by Secure Cloud Connector"
			check_sshd_permit_tunnel $SSHD_CONFIG
        		exit 0;
		else
        		echo "OpenSSH version $CURR_VER  at $SSHD_BIN is NOT supported by Secure Cloud Connector"
		fi
	else
		echo "$SSHD_BIN was not found"
	fi
}

check_config_tun
check_sshd_version "/usr/sbin/sshd" "/etc/ssh/sshd_config"
check_sshd_version "/usr/local/sbin/sshd" "/usr/local/etc/sshd_config"

OPENSSH_VER="5.3p1"
OPENSSL_VER=`rpm -q openssl | cut -d '-' -f 2`
OPENSSL_DEVEL_VER=`rpm -q openssl-devel | cut -d '-' -f 3`
ZLIB_VER=`rpm -q zlib | cut -d '-' -f 2`
ZLIB_DEVEL_VER=`rpm -q zlib-devel | cut -d '-' -f 3`

NOTFOUND=
echo "Download the following sources from their websites into the 'sources' directory"
mkdir $WD/sources
if [ ! -f sources/openssh-${OPENSSH_VER}.tar.gz ]; then
	echo "OpenSSH: http://ftp5.usa.openbsd.org/pub/OpenBSD/OpenSSH/portable/openssh-${OPENSSH_VER}.tar.gz"
	NOTFOUND=1
fi
if [ "X$OPENSSL_DEVEL_VER" == "X" ] && [ ! -f sources/openssl-${OPENSSL_VER}.tar.gz ]; then
	echo "OpenSSL: http://openssl.org/source/openssl-${OPENSSL_VER}.tar.gz"
	NOTFOUND=1
fi
if [ "X$ZLIB_DEVEL_VER" == "X" ] && [ ! -f sources/zlib-${ZLIB_VER}.tar.gz ] ; then
	echo "Zlib: http://zlib.net/fossils/zlib-${ZLIB_VER}.tar.gz"
	NOTFOUND=1
fi

if [ "X$NOTFOUND" != "X" ]; then
	echo "Download the required files and re-run the build"
	exit -1;
fi
#
mkdir $WD/build
cd $WD/build

if [ "X$ZLIB_DEVEL_VER" == "X" ]; then
	echo "Building zlib-${ZLIB_VER}"
	tar -xzf $WD/sources/zlib-${ZLIB_VER}.tar.gz
	cd zlib-${ZLIB_VER}
	./configure  > config1.log
	if [ "$?" -ne 0 ]; then
		echo "configure failed"
		exit -1
	fi
	make
	if [ "$?" -ne 0 ]; then
		echo "Make failed"
		exit -1
	fi
	make install
	if [ "$?" -ne 0 ]; then
		echo "Make install failed"
		exit -1
	fi
fi

cd $WD/build

if [ "X$OPENSSL_DEVEL_VER" == "X" ]; then
	echo "Building openssl-${OPENSSL_VER}"
	tar -xzf $WD/sources/openssl-${OPENSSL_VER}.tar.gz
	cd openssl-${OPENSSL_VER}
	./config > config1.log
	if [ "$?" -ne 0 ]; then
		echo "configure failed"
		exit -1
	fi
	make
	if [ "$?" -ne 0 ]; then
		echo "Make failed"
		exit -1
	fi
	make install
	if [ "$?" -ne 0 ]; then
		echo "Make install failed"
		exit -1
	fi
fi

cd $WD/build
echo "Building openssh-${OPENSSH_VER}"
tar -xzvf "$WD/sources/openssh-${OPENSSH_VER}.tar.gz"
cd openssh-${OPENSSH_VER}
./configure
if [ "$?" -ne 0 ]; then
	echo "configure failed"
	exit -1
fi
make 
if [ "$?" -ne 0 ]; then
	echo "make failed"
	exit -1
fi
make install
if [ "$?" -ne 0 ]; then
	echo "make install failed"
	exit -1
fi

mv /usr/local/etc/sshd_config /usr/local/etc/sshd_config.orig
cp $WD/sshd_config  /usr/local/etc/
