<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dp="http://www.datapower.com/extensions"
    extension-element-prefixes="dp">
  <dp:output-mapping href="store:///pkcs7-convert-input.ffd"/>

  <xsl:template match="/">
    <xsl:variable name="all-files"
        select="dp:variable('var://local/attachment-manifest')/manifest/attachments/attachment/header"/>

    <xsl:for-each select="$all-files/../uri">
      <!-- determine the new attachment uri -->
      <xsl:variable name="nuri"
          select="concat('attachment://INPUT/cid:archive-zip',
                         '?Archive=zip',
                         '&amp;Filename=', substring-after(., 'cid:'))"/>

      <!-- add to archive-zip -->
      <dp:url-open target="{$nuri}" data-type="base64" response="ignore">
        <!-- read the current attachment as binaryNode -->
        <xsl:variable name="resp">
          <dp:url-open target="{.}" response="binaryNode"/>
        </xsl:variable>

        <!-- The only safe way to pass binary data to dp:url-open is by
             base64 encoding it and 'data-type="base64"' in dp:url-open -->
        <xsl:value-of select="dp:binary-encode($resp/result/binary/node())"/>
      </dp:url-open>
    </xsl:for-each>

    <!-- Now that archive-zip contains all files needed.
         Extract it as binaryNode for sending to output. -->
    <xsl:variable name="zip">
      <dp:url-open target="attachment://INPUT/cid:archive-zip" response="binaryNode"/>
    </xsl:variable>

    <!-- cleanup -->
    <dp:strip-attachments/>

    <!-- prepare for dp:output-mapping -->
    <object>
      <message>
        <xsl:copy-of select="$zip/result/binary/node()"/>
      </message>
    </object>
  </xsl:template>
</xsl:stylesheet>

